@@ -1,13 +1,13 @@
 ;;; bible-gateway.el --- A Simple BibleGateway Client -*- lexical-binding: t -*-
 
-;; Copyright (C) 2025 Kristjon Ciko
+;; Copyright (C) 2026 Kristjon Ciko
 
 ;; Author: Kristjon Ciko
 ;; Keywords: convenience comm hypermedia
 ;; Homepage: https://github.com/kristjoc/bible-gateway
 ;; Package-Requires: ((emacs "29.1"))
-;; Package-Version: 20251008.1132
-;; Package-Revision: fb79e01beb9b
+;; Package-Version: 20260105.1259
+;; Package-Revision: 8616ccb52a37
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -33,20 +33,19 @@
 ;; `bible-gateway-get-verse' fetches the verse of the day for use as
 ;; an emacs-dashboard footer or a scratch buffer message.
 ;;
-;; `bible-gateway-get-passage' fetches a specific passage and inserts
-;; it at point. It can be called both interactively from M-x or
-;; programmatically with the book name and verse(s) as arguments.
+;; M-x `bible-gateway-get-passage' fetches a Bible passage and inserts
+;; it at point. It can be called both interactively from
+;; \\[execute-extended-command] or programmatically with the book name
+;; and verse(s) as arguments.
 ;;
-;; M-x `bible-gateway-listen-passage-in-browser' plays a specific
-;; audio chapter in the browser.
-;;
-;; M-x `bible-gateway-listen-passage-with-emms' plays a specific audio
-;; chapter using EMMS.
+;; M-x `bible-gateway-listen-passage' plays a Bible chapter from KJV
+;; Zondervan Audio in the browser.
 
 ;;; Code:
 
 (require 'url)
 (require 'json)
+(require 'cl-lib)
 
 (defgroup bible-gateway nil
   "Package that fetches the Bible verse of the day from BibleGateway."
@@ -58,38 +57,33 @@
 
 (defcustom bible-gateway-bible-version "KJV"
   "The Bible version, default KJV.
-Other supported versions, which are available in the Public domain, are
-LSG in French, RVA in Spanish, ALB in Albanian, and UKR in Ukrainian."
-  :type 'string
-  :group 'bible-gateway)
+Other supported versions, which are available in the Public Domain, are
+LSG in French, RVA in Spanish, ALB in Albanian, UKR in Ukrainian, RUSV
+in Russian, and LUTH1545 in German."
+  :type 'string)
 
 (defcustom bible-gateway-text-width 80
   "The width of the verse of the day body in number of characters, default 80."
-  :type 'integer
-  :group 'bible-gateway)
+  :type 'natnum)
 
 (defcustom bible-gateway-fallback-verse "For God so loved the world,
 that he gave his only begotten Son,
 that whosoever believeth in him should not perish,
 but have everlasting life."
   "The fallback verse to use when the online request fails."
-  :type 'string
-  :group 'bible-gateway)
+  :type 'string)
 
 (defcustom bible-gateway-fallback-reference "John 3:16"
   "The reference for the fallback verse."
-  :type 'string
-  :group 'bible-gateway)
+  :type 'string)
 
 (defcustom bible-gateway-request-timeout 3
   "The timeout for the URL request in seconds."
-  :type 'integer
-  :group 'bible-gateway)
+  :type 'integer)
 
 (defcustom bible-gateway-include-ref t
   "If non-nil, print the reference (e.g., \"John 3 (KJV)\") with the passage."
-  :type 'boolean
-  :group 'bible-gateway)
+  :type 'boolean)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;                     Bible books in supported languages                     ;
@@ -195,11 +189,90 @@ but have everlasting life."
     ("2 Івана" . 1) ("3 Івана" . 1) ("Юда" . 1) ("Об'явлення" . 22))
   "List of Bible books (UKR version) with their number of chapters.")
 
+(defconst bible-gateway-bible-books-rusv
+  '(("Бытие" . 50) ("Исход" . 40) ("Левит" . 27) ("Числа" . 36)
+    ("Второзаконие" . 34) ("Иисус Навин" . 24) ("Книга Судей" . 21) ("Руфь" . 4)
+    ("1-я Царств" . 31) ("2-я Царств" . 24) ("3-я Царств" . 22) ("4-я Царств" . 25)
+    ("1-я Паралипоменон" . 29) ("2-я Паралипоменон" . 36) ("Ездра" . 10) ("Неемия" . 13)
+    ("Есфирь" . 10) ("Иов" . 42) ("Псалтирь" . 150) ("Притчи" . 31)
+    ("Екклесиаст" . 12) ("Песни Песней" . 8) ("Исаия" . 66) ("Иеремия" . 52)
+    ("Плач Иеремии" . 5) ("Иезекииль" . 48) ("Даниил" . 12) ("Осия" . 14)
+    ("Иоиль" . 3) ("Амос" . 9) ("Авдия" . 1) ("Иона" . 4) ("Михей" . 7)
+    ("Наум" . 3) ("Аввакум" . 3) ("Софония" . 3) ("Аггей" . 2)
+    ("Захария" . 14) ("Малахия" . 4) ("От Матфея" . 28) ("От Марка" . 16)
+    ("От Луки" . 24) ("От Иоанна" . 21) ("Деяния" . 28) ("К Римлянам" . 16)
+    ("1-е Коринфянам" . 16) ("2-е Коринфянам" . 13) ("К Галатам" . 6)
+    ("К Ефесянам" . 6) ("К Филиппийцам" . 4) ("К Колоссянам" . 4)
+    ("1-е Фессалоникийцам" . 5) ("2-е Фессалоникийцам" . 3) ("1-е Тимофею" . 6)
+    ("2-е Тимофею" . 4) ("К Титу" . 3) ("К Филимону" . 1) ("К Евреям" . 13)
+    ("Иакова" . 5) ("1-e Петра" . 5) ("2-e Петра" . 3) ("1-e Иоанна" . 5)
+    ("2-e Иоанна" . 1) ("3-e Иоанна" . 1) ("Иуда" . 1) ("Откровение" . 22))
+  "List of Bible books (RUSV version) with their number of chapters.")
+
+(defconst bible-gateway-bible-books-luth1545
+  '(("1 Mose" . 50) ("2 Mose" . 40) ("3 Mose" . 27) ("4 Mose" . 36)
+    ("5 Mose" . 34) ("Josua" . 24) ("Richter" . 21) ("Rut" . 4)
+    ("1 Samuel" . 31) ("2 Samuel" . 24) ("1 Koenige" . 22) ("2 Koenige" . 25)
+    ("1 Chronik" . 29) ("2 Chronik" . 36) ("Esra" . 10) ("Nehemia" . 13)
+    ("Ester" . 10) ("Hiob" . 42) ("Psalm" . 150) ("Sprueche" . 31)
+    ("Prediger" . 12) ("Hohelied" . 8) ("Jesaja" . 66) ("Jeremia" . 52)
+    ("Klagelieder" . 5) ("Hesekiel" . 48) ("Daniel" . 12) ("Hosea" . 14)
+    ("Joel" . 3) ("Amos" . 9) ("Obadja" . 1) ("Jona" . 4) ("Mica" . 7)
+    ("Nahum" . 3) ("Habakuk" . 3) ("Zephanja" . 3) ("Hagai" . 2)
+    ("Sacharja" . 14) ("Maleachi" . 4) ("Matthaeus" . 28) ("Markus" . 16)
+    ("Lukas" . 24) ("Johannes" . 21) ("Apostelgeschichte" . 28) ("Roemer" . 16)
+    ("1 Korinther" . 16) ("2 Korinther" . 13) ("Galater" . 6)
+    ("Epheser" . 6) ("Philipper" . 4) ("Kolosser" . 4)
+    ("1 Thessalonicher" . 5) ("2 Thessalonicher" . 3) ("1 Timotheus" . 6)
+    ("2 Timotheus" . 4) ("Titus" . 3) ("Philemon" . 1) ("Hebraeer" . 13)
+    ("Jakobus" . 5) ("1 Petrus" . 5) ("2 Petrus" . 3) ("1 Johannes" . 5)
+    ("2 Johannes" . 1) ("3 Johannes" . 1) ("Judas" . 1) ("Offenbarung" . 22))
+  "List of Bible books (Luther Bibel 1545 version) with their number of chapters.")
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;                           Caching Mechanism                                ;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defvar bible-gateway-cache-dir
+  (locate-user-emacs-file ".cache/bible-gateway/")
+  "Directory where bible-gateway cache file is stored.")
+
+(defvar bible-gateway-cache-file
+  (expand-file-name "bible-gateway-votd" bible-gateway-cache-dir)
+  "File path for the verse of the day cache.")
+
+(defun bible-gateway--ensure-cache-dir ()
+  "Ensure that the cache directory exists."
+  (unless (file-exists-p bible-gateway-cache-dir)
+    (make-directory bible-gateway-cache-dir t)))
+
+(defun bible-gateway--save-cache (data date)
+  "Save the DATA (formatted string) and DATE to the cache file."
+  (bible-gateway--ensure-cache-dir)
+  (with-temp-file bible-gateway-cache-file
+    (let ((print-length nil)
+          (print-level nil))
+      (insert ";; Bible Gateway Verse of the Day Cache\n")
+      (prin1 `(:date ,date :data ,data) (current-buffer)))))
+
+(defun bible-gateway--read-cache ()
+  "Read and return the cached plist (:date :data).
+Returns nil if the cache file does not exist or is invalid."
+  (when (file-exists-p bible-gateway-cache-file)
+    (condition-case nil
+        (with-temp-buffer
+          (insert-file-contents bible-gateway-cache-file)
+          (goto-char (point-min))
+          (read (current-buffer)))
+      (error nil))))
+
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;                     Package Section I: Fetch the Verse of The Day          ;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defun bible-gateway-justify-line (line width)
+(defun bible-gateway--justify-line (line width)
   "Justify LINE to WIDTH characters."
   (let* ((words (split-string line))
          (word-count (length words)))
@@ -223,7 +296,7 @@ but have everlasting life."
                          ?\s))))
         (concat result (car (last words)))))))
 
-(defun bible-gateway-format-verse-text (text &optional width)
+(defun bible-gateway--format-verse (text &optional width)
   "Format verse TEXT as a justified paragraph with optional WIDTH."
   (when text  ; Only process if text is not nil
     (with-temp-buffer
@@ -245,83 +318,87 @@ but have everlasting life."
         (let* ((justified-lines
                 (append
                  (mapcar (lambda (line)
-                           (bible-gateway-justify-line line fill-column))
+                           (bible-gateway--justify-line line fill-column))
                          (butlast lines))
                  (last lines)))
                (result (string-join justified-lines "\n")))
           result)))))
 
-(defun bible-gateway-decode-html-entities (text)
+(defun bible-gateway--decode-html (text)
   "Decode HTML entities in TEXT, including numeric character references."
   (when text
     (let ((entity-map '(("&ldquo;" . "\"")
-			("&rdquo;" . "\"")
-			("&#8212;" . "--")
-			("&#8217;" . "'")
-			("&#8220;" . "\"")
-			("&#8221;" . "\"")
-			("&quot;" . "\"")
-			("&apos;" . "'")
-			("&lt;" . "<")
-			("&gt;" . ">")
-			("&nbsp;" . " ")
-			("&amp;" . "&") ; Decode &amp; first
-			("&#039;" . "'")))) ; Decode numeric entities like &#039; after &amp;
-      ;; Replace named entities
-      (dolist (entity entity-map text)
-        (setq text (replace-regexp-in-string (car entity) (cdr entity) text)))
-      ;; Replace numeric character references (e.g., &#233; -> é)
-      ;; Fix for special characters in LSG and RVA
-      (setq text
-	    (replace-regexp-in-string "&#\\([0-9]+\\);"
-                                      (lambda (match)
-                                        (char-to-string (string-to-number (match-string 1 match))))
-                                      text))
-      ;; Return the decoded text
-      text)))
+                        ("&rdquo;" .  "\"")
+                        ("&#8212;" . "—")
+                        ("&#8217;" . "'")
+                        ("&#8220;" . "\"")
+                        ("&#8221;" . "\"")
+                        ("&quot;" . "\"")
+                        ("&apos;" .  "'")
+                        ("&lt;" . "<")
+                        ("&gt;" . ">")
+                        ("&nbsp;" . " ")
+                        ("&amp;" . "&"))))
+      (replace-regexp-in-string
+       "&#\\([0-9]+\\);"
+       (lambda (match)
+	 (char-to-string (string-to-number (match-string 1 match))))
+       (replace-regexp-in-string
+	"&[a-z]+;"
+	(lambda (match)
+	  (or (alist-get match entity-map nil nil #'string=)
+	      match))
+	text)))))
 
-(defun bible-gateway-fetch-daily-bible-verse ()
-  "Fetch the daily Bible verse from BibleGateway API."
+(defun bible-gateway--fetch-votd ()
+  "Fetch the daily Bible verse using the BibleGateway API.
+If the API is unavailable (e.g., geo-blocked), falls back to scraping.
+If scraping also fails, returns the fallback verse."
   (let ((url-request-method "GET")
-        (url (concat "https://www.biblegateway.com/votd/get/?format=json&version=" bible-gateway-bible-version)))
+        (url (concat "https://www.biblegateway.com/votd/get/?format=json&version=" bible-gateway-bible-version))
+        (fallback-result (format "%s\n%s"
+                                 (bible-gateway--format-verse bible-gateway-fallback-verse)
+                                 (let ((ref-text bible-gateway-fallback-reference))
+                                   (concat (make-string (- bible-gateway-text-width (length ref-text)) ?\s)
+                                           ref-text)))))
     (condition-case nil
-        (with-current-buffer (url-retrieve-synchronously url t t bible-gateway-request-timeout)
-          (goto-char (point-min))
-          (when (search-forward "\n\n" nil t)
-            (let* ((json-string (buffer-substring-no-properties (point) (point-max)))
-                   (json-object-type 'hash-table)
-                   (json-array-type 'list)
-                   (json-key-type 'string)
-                   (json-data (json-read-from-string json-string))
-                   (votd (gethash "votd" json-data))
-                   (raw-text (gethash "text" votd))
-                   (verse-text (bible-gateway-decode-html-entities raw-text))
-                   (clean-verse (replace-regexp-in-string "[\"]" "" verse-text))
-                   (formatted-verse (bible-gateway-format-verse-text clean-verse))
-                   (verse-reference (gethash "display_ref" votd))
-                   (fill-width bible-gateway-text-width))
-              (format "%s\n%s"
-                      formatted-verse
-                      (let ((ref-text verse-reference))
-                        (concat (make-string (- fill-width (length ref-text)) ?\s)
-                                ref-text))))))
+	(with-current-buffer (let ((url-mime-charset-string "utf-8"))
+			       (url-retrieve-synchronously url t t bible-gateway-request-timeout))
+	  (goto-char (point-min))
+	  (when (search-forward "\n\n" nil t)
+	    (let ((response-body (buffer-substring-no-properties (point) (point-max))))
+              ;; Check if API returned "Content Unavailable" HTML instead of JSON
+	      (if (or (string-match-p "<title>Content Unavailable</title>" response-body)
+		      (not (string-match-p "^\\s-*{" response-body)))
+                  ;; API unavailable, try scraping
+                  (condition-case nil
+                      (bible-gateway--scrape-votd)
+                    (error
+                     (message "BibleGateway API and scraping unavailable, using fallback verse.")
+                     fallback-result))
+                ;; API returned JSON, process it
+                (let* ((json-data (json-parse-string response-body :object-type 'hash-table))
+                       (votd (gethash "votd" json-data))
+                       (raw-text (gethash "text" votd))
+                       (verse-text (bible-gateway--decode-html raw-text))
+                       (clean-verse (replace-regexp-in-string "[\"]" "" verse-text))
+                       (formatted-verse (bible-gateway--format-verse clean-verse))
+                       (verse-reference (gethash "display_ref" votd))
+                       (fill-width bible-gateway-text-width))
+                  (format "%s\n%s"
+                          formatted-verse
+                          (let ((ref-text verse-reference))
+                            (concat (make-string (- fill-width (length ref-text)) ?\s)
+                                    ref-text))))))))
       (error
-       (message "%s\n%s"
-                (bible-gateway-format-verse-text bible-gateway-fallback-verse)
-                (let ((ref-text bible-gateway-fallback-reference))
-                  (concat (make-string (- bible-gateway-text-width (length ref-text)) ?\s)
-                          ref-text)))))))
+       ;; Network error or timeout, try scraping first
+       (condition-case nil
+           (bible-gateway--scrape-votd)
+         (error
+          (message "BibleGateway unreachable, using fallback verse.")
+          fallback-result))))))
 
-;;;###autoload
-(defun bible-gateway-get-verse ()
-  "Get the daily verse and handle errors."
-  (condition-case err
-      (bible-gateway-fetch-daily-bible-verse)
-    (error
-     (message "Today's verse could not be fetched: %s" (error-message-string err)))))
-
-;;;###autoload
-(defun bible-gateway-get-verse-in-eu ()
+(defun bible-gateway--scrape-votd ()
   "Fetch the Verse of the Day by scraping BibleGateway.
 Returns a single formatted string without verse numbers nor reference."
   (condition-case _err
@@ -329,6 +406,8 @@ Returns a single formatted string without verse numbers nor reference."
             citation book passage)
         ;; 1. Retrieve homepage and extract citation.
         (with-current-buffer (url-retrieve-synchronously url t t bible-gateway-request-timeout)
+	  (set-buffer-multibyte t)
+	  (decode-coding-region (point-min) (point-max) 'utf-8)
           (goto-char (point-min))
           (when (re-search-forward "<span class=\"citation\">\\([^<]+\\)</span>" nil t)
             (setq citation (match-string 1))))
@@ -341,7 +420,8 @@ Returns a single formatted string without verse numbers nor reference."
                (book (string-join (butlast parts) " ")))
           ;; 3. Capture passage output.
           (with-temp-buffer
-            (bible-gateway-get-passage book passage)
+	    (let ((bible-gateway-include-ref nil))
+              (bible-gateway-get-passage book passage))
             (let* ((raw (buffer-string))
                    ;; Remove any success/status lines.
                    (raw (replace-regexp-in-string "Bible passage inserted successfully!.*$" "" raw))
@@ -364,7 +444,7 @@ Returns a single formatted string without verse numbers nor reference."
                   (unless skip
                     ;; Remove verse numbers.
                     (setq trim (replace-regexp-in-string "\\s-*\\([0-9]+\\)\\.\\s-*" " " trim))
-                    (when (> (length trim) 0)
+		    (when (length> trim 0)
                       (push trim kept)))))
 
               (setq kept (nreverse kept))
@@ -377,7 +457,7 @@ Returns a single formatted string without verse numbers nor reference."
                      (joined (string-trim joined)))
 
                 ;; 6. Final format with your existing formatter.
-                (let* ((formatted (bible-gateway-format-verse-text joined))
+                (let* ((formatted (bible-gateway--format-verse joined))
                        (ref-text (format "%s (%s)" citation bible-gateway-bible-version))
                        (fill-width bible-gateway-text-width)
                        (aligned-ref (concat (make-string (max 0 (- fill-width (length ref-text))) ?\s)
@@ -385,62 +465,80 @@ Returns a single formatted string without verse numbers nor reference."
                   (format "%s\n%s" formatted aligned-ref)))))))
     (error
      ;; Fallback verse.
-     (let* ((formatted (bible-gateway-format-verse-text bible-gateway-fallback-verse))
+     (let* ((formatted (bible-gateway--format-verse bible-gateway-fallback-verse))
             (ref-text bible-gateway-fallback-reference)
             (aligned-ref (concat (make-string (max 0 (- bible-gateway-text-width (length ref-text))) ?\s)
                                  ref-text)))
        (format "%s\n%s" formatted aligned-ref)))))
 
+
+;;;###autoload
+(defun bible-gateway-get-verse ()
+  "Get the verse of the day.
+Checks the cache first. If the cache is from today, returns the cached
+string. Otherwise, fetches the verse from BibleGateway, updates the
+cache ONLY if successful, and returns the verse."
+  (let* ((today (format-time-string "%Y-%m-%d"))
+         (cached-item (bible-gateway--read-cache))
+         (cached-date (plist-get cached-item :date)))
+
+    (if (string= today cached-date)
+        ;; Cache Hit: Return the stored formatted string
+        (plist-get cached-item :data)
+
+      ;; Cache Miss: Fetch
+      (let ((result (bible-gateway--fetch-votd)))
+        ;; Check if result is the fallback verse (formatted). We
+        ;; construct the fallback string exactly as the original
+        ;; function does to compare.
+        (let ((fallback-string (format
+				"%s\n%s" (bible-gateway--format-verse
+					  bible-gateway-fallback-verse)
+				(let ((ref-text bible-gateway-fallback-reference))
+				  (concat (make-string (- bible-gateway-text-width
+							  (length ref-text))
+						       ?\s)
+					  ref-text)))))
+
+          ;; Only save to cache if it is NOT the fallback verse
+          (unless (string= result fallback-string)
+            (bible-gateway--save-cache result today))
+
+          ;; Return the result (whether real or fallback)
+          result)))))
+
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;                     Package section II - Fetch a Bible Passage             ;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defun bible-gateway-read-book ()
-  "Read a Bible book name with completion based on selected Bible version."
+(defun bible-gateway--prompt-book ()
+  "Prompt for a Bible book name with completion based on selected Bible version."
   (completing-read
    "Select a Book: "
    (mapcar #'car  ; Get just the book names for completion
-           (cond ((string= bible-gateway-bible-version "KJV")
-                  bible-gateway-bible-books-kjv)
-                 ((string= bible-gateway-bible-version "LSG")
-                  bible-gateway-bible-books-lsg)
-                 ((string= bible-gateway-bible-version "RVA")
-                  bible-gateway-bible-books-rva)
-		 ((string= bible-gateway-bible-version "ALB")
-		  bible-gateway-bible-books-alb)
-		 ((string= bible-gateway-bible-version "UKR")
-		  bible-gateway-bible-books-ukr)
-                 (t bible-gateway-bible-books-kjv)))
+	   (pcase bible-gateway-bible-version
+	     ("KJV" bible-gateway-bible-books-kjv)
+	     ("LSG" bible-gateway-bible-books-lsg)
+	     ("RVA" bible-gateway-bible-books-rva)
+	     ("ALB" bible-gateway-bible-books-alb)
+	     ("UKR" bible-gateway-bible-books-ukr)
+	     ("RUSV" bible-gateway-bible-books-rusv)
+	     ("LUTH1545" bible-gateway-bible-books-luth1545)
+	     (_ bible-gateway-bible-books-kjv)))
    nil t))
 
-(defun bible-gateway-get-chapter-count (book version)
-  "Get the number of chapters for BOOK in the given VERSION."
-  (let ((books-list (cond ((string= version "KJV")
-                           bible-gateway-bible-books-kjv)
-                          ((string= version "LSG")
-                           bible-gateway-bible-books-lsg)
-                          ((string= version "RVA")
-                           bible-gateway-bible-books-rva)
-			  ((string= version "ALB")
-                           bible-gateway-bible-books-alb)
-                          ((string= version "UKR")
-                           bible-gateway-bible-books-ukr)
-                          (t bible-gateway-bible-books-kjv))))
-    (cdr (assoc book books-list))))
-
-(defun bible-gateway-read-chapter-verse (book)
-  "Read chapter and verse for BOOK."
-  (let* ((books-list (cond ((string= bible-gateway-bible-version "KJV")
-                            bible-gateway-bible-books-kjv)
-                           ((string= bible-gateway-bible-version "LSG")
-                            bible-gateway-bible-books-lsg)
-                           ((string= bible-gateway-bible-version "RVA")
-                            bible-gateway-bible-books-rva)
-                           ((string= bible-gateway-bible-version "ALB")
-                            bible-gateway-bible-books-alb)
-                           ((string= bible-gateway-bible-version "UKR")
-                            bible-gateway-bible-books-ukr)
-                           (t bible-gateway-bible-books-kjv)))
+(defun bible-gateway--prompt-chapter-verse (book)
+  "Prompt for a chapter and verse for the BOOK."
+  (let* ((books-list (pcase bible-gateway-bible-version
+		       ("KJV" bible-gateway-bible-books-kjv)
+		       ("LSG" bible-gateway-bible-books-lsg)
+		       ("RVA" bible-gateway-bible-books-rva)
+		       ("ALB" bible-gateway-bible-books-alb)
+		       ("UKR" bible-gateway-bible-books-ukr)
+		       ("RUSV" bible-gateway-bible-books-rusv)
+		       ("LUTH1545" bible-gateway-bible-books-luth1545)
+		       (_ bible-gateway-bible-books-kjv)))
          (max-chapters (cdr (assoc book books-list))))
     (unless max-chapters
       (error "Could not find chapter count for book: %s in version %s" book bible-gateway-bible-version))
@@ -448,46 +546,35 @@ Returns a single formatted string without verse numbers nor reference."
                   (format "Select passage from %s (1-%d): " book max-chapters))))
       (format "%s %s" book input))))
 
-(defun bible-gateway-process-verse-text (text)
+(defun bible-gateway--process-verse-text (text)
   "Process verse TEXT.
-Handling special cases like small-caps LORD and UTF-8 encoding."
-  (let ((processed-text text))
-    ;; First decode UTF-8 octal sequences
-    (setq processed-text
-          (decode-coding-string
-           (encode-coding-string processed-text 'utf-8)
-	   'utf-8))
+Handling special cases like small-caps LORD or JESUS and UTF-8 encoding."
+  (with-temp-buffer
+    (insert text)
+    (cl-flet ((replace-all (pattern replacement)
+                (goto-char (point-min))
+                (while (re-search-forward pattern nil t)
+                  (replace-match replacement t t))))
+      ;; Replace small-caps Lord with "LORD"
+      (replace-all "<span style=\"font-variant: small-caps\" class=\"small-caps\">\\(Lord\\)</span>" "LORD")
+      ;; Replace small-caps span with "JESUS"
+      (replace-all "<span style=\"font-variant: small-caps\" class=\"small-caps\">\\(Jesus\\)</span>" "JESUS")
+      ;; Remove "Read full chapter" text
+      (replace-all "Read full chapter.*$" "")
+      ;; Remove empty span tags (like <span class="text Rev-22-21"></span>)
+      (replace-all "<span[^>]*>\\s-*</span>" "")
+      ;; Remove trailing span IDs and div tags
+      (replace-all "\\(<span id=\"[^\"]*\".*\\|</div.*\\)$" "")
+      ;; Remove incomplete HTML tags at the end (like <span without >)
+      (replace-all "<[^>]*$" "")
+      ;; Remove any remaining HTML tags
+      (replace-all "<[^>]+>" "")
+      ;; Fix non-breaking space
+      (replace-all "\\\\302\\\\240" " "))
+    ;; Decode HTML entities (if any remain) and trim
+    (string-trim (bible-gateway--decode-html (buffer-string)))))
 
-    ;; Replace small-caps span with "LORD"
-    (setq processed-text
-          (replace-regexp-in-string
-           "<span style=\"font-variant: small-caps\" class=\"small-caps\">\\(Lord\\)</span>"
-           "LORD"
-           processed-text t))
-
-    ;; Remove "Read full chapter" text
-    (setq processed-text
-          (replace-regexp-in-string "Read full chapter.*$" "" processed-text))
-
-    ;; Remove trailing span IDs and div tags
-    (setq processed-text
-          (replace-regexp-in-string "\\(<span id=\"[^\"]*\".*\\|</div.*\\)$" "" processed-text))
-
-    ;; Remove any remaining HTML tags
-    (setq processed-text
-          (replace-regexp-in-string "<[^>]+>" "" processed-text))
-
-    ;; Fix non-breaking space
-    (setq processed-text
-          (replace-regexp-in-string "\\\\302\\\\240" " " processed-text))
-
-    ;; Decode HTML entities (if any remain)
-    (setq processed-text (bible-gateway-decode-html-entities processed-text))
-
-    ;; Trim whitespace and return
-    (string-trim processed-text)))
-
-(defun bible-gateway-wrap-verse-text (verse-text)
+(defun bible-gateway--wrap-verse-text (verse-text)
   "Wrap VERSE-TEXT according to window width with 4 spaces for wrapped lines."
   (with-temp-buffer
     (insert verse-text)
@@ -505,21 +592,23 @@ If neither, prompt for both."
   (interactive)
   (let* ((book-supplied (and book (not (string-empty-p book))))
          (passage-supplied (and passage (not (string-empty-p passage))))
-         (chosen-book (if book-supplied book (bible-gateway-read-book)))
+         (chosen-book (if book-supplied book (bible-gateway--prompt-book)))
          (chosen-passage (if passage-supplied
                              passage
-                           (bible-gateway-read-chapter-verse chosen-book)))
+                           (bible-gateway--prompt-chapter-verse chosen-book)))
          (search-string (if passage-supplied
                             (concat chosen-book
 				    (replace-regexp-in-string " " "" chosen-passage))
                           (replace-regexp-in-string " " "" chosen-passage)))
          (url (concat "https://www.biblegateway.com/passage/?search="
-                      search-string
-                      "&version=" bible-gateway-bible-version)))
+		      (url-encode-url search-string)
+		      "&version=" (url-encode-url bible-gateway-bible-version))))
     (condition-case err
-        (let ((original-buffer (current-buffer)))
-          (with-current-buffer
+	(let ((original-buffer (current-buffer)))
+	  (with-current-buffer
 	      (url-retrieve-synchronously url t t bible-gateway-request-timeout)
+	    (set-buffer-multibyte t)
+	    (decode-coding-region (point-min) (point-max) 'utf-8)
             (goto-char (point-min))
 
 	    ;; First get the title if required
@@ -538,10 +627,10 @@ If neither, prompt for both."
               ;; Then get the verses
               (goto-char (point-min))
               (if (search-forward "<div class='passage-content passage-class-0'>" nil t)
-                  (let* ((start (point))
-                         (end (search-forward "</div>"))
-                         (raw-content (buffer-substring-no-properties start (1- end)))
-                         (verses '()))
+		  (let* ((start (point))
+			 (end (search-forward "</div>"))
+			 (raw-content (buffer-substring-no-properties start (1- end)))
+			 (verses '()))
                     ;; (pos 0))
 
                     ;; First, remove chapter numbers
@@ -568,9 +657,9 @@ If neither, prompt for both."
                                (verse-content
 				(buffer-substring-no-properties verse-start verse-end))
 			       ;; First process the verse text normally
-                               (verse-text (bible-gateway-process-verse-text verse-content))
+                               (verse-text (bible-gateway--process-verse-text verse-content))
 			       ;; Then wrap it with proper formatting
-			       (final-text (bible-gateway-wrap-verse-text verse-text)))
+			       (final-text (bible-gateway--wrap-verse-text verse-text)))
 			  (when (not (string-empty-p final-text))
 			    (push (format "%s.%s%s" verse-num
 					  (if (< (string-to-number verse-num) 10) "  " " ")
@@ -589,7 +678,7 @@ Please double-check that the chapter and verse numbers are valid."))))))
        (message "Error while fetching the passage: %s" (error-message-string err))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;                    Package Section III - Play Audio chapter                ;
+;;   Package Section III - Play Bible chapter from KJV Dramatized Audio       ;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (defconst bible-gateway-bible-books-osis
@@ -618,20 +707,19 @@ Please double-check that the chapter and verse numbers are valid."))))))
     ("3 John" . "3John") ("Jude" . "Jude") ("Revelation" . "Rev"))
   "Mapping of Bible book names to their OSIS abbreviations for audio links.")
 
-(defun bible-gateway-get-audio-link (book chapter)
-  "Generate and open BibleGateway audio link for BOOK CHAPTER and optional VERSE."
+(defun bible-gateway--get-audio-link (book chapter)
+  "Generate and open BibleGateway audio link for CHAPTER from BOOK."
   (let* ((osis-code (cdr (assoc book bible-gateway-bible-books-osis)))
          (version (downcase bible-gateway-bible-version))
          (base-url "https://www.biblegateway.com/audio/dramatized")
          (url (if osis-code
                   (format "%s/%s/%s.%d" base-url version osis-code chapter)
 		(error "Unknown book: %s" book))))
-    ;; Return URL for display
     url))
 
 ;;;###autoload
-(defun bible-gateway-listen-passage-in-browser ()
-  "Open a browser tab to listen the requested chapter."
+(defun bible-gateway-listen-passage ()
+  "Prompt for a Bible chapter and play KJV Dramatized Audio in the browser."
   (interactive)
   (let* ((books-list (cond ((string= bible-gateway-bible-version "KJV")
                             bible-gateway-bible-books-kjv)
@@ -640,73 +728,13 @@ Please double-check that the chapter and verse numbers are valid."))))))
          (max-chapters (cdr (assoc book books-list)))
          (input (read-string
                  (format "Select Chapter from %s (1-%d): " book max-chapters)))
-         (audio-link (bible-gateway-get-audio-link book (string-to-number input))))
+         (audio-link (bible-gateway--get-audio-link book
+						    (string-to-number input))))
     (browse-url audio-link)
     ;; More detailed message with the specific chapter being opened
     (message "Switch to your browser and click Play to listen %s %s (KJV)."
              book input)))
 
-;; (defun bible-gateway-listen-passage-with-emms ()
-;;   "Download and play the requested Bible chapter audio with EMMS."
-;;   (interactive)
-;;   (let* ((books-list (cond ((string= bible-gateway-bible-version "KJV")
-;;                             bible-gateway-bible-books-kjv)
-;;                            (t bible-gateway-bible-books-kjv)))
-;;          (book (completing-read "Select Book: " (mapcar #'car books-list)))
-;;          (max-chapters (cdr (assoc book books-list)))
-;;          (input (read-string
-;;                  (format "Select Chapter from %s (1-%d): " book max-chapters)))
-;;          ;; Get the audio link but don't open it in a browser
-;;          (audio-link (let ((browse-url-browser-function #'ignore)
-;;                            (url-show-status nil)) ; Silence "Contacting host..." messages
-;;                        (bible-gateway-get-audio-link book (string-to-number input))))
-;; 	 (output-file (expand-file-name (concat book "-" input ".mp3")
-;; 					(temporary-file-directory)))
-;;          (temp-html-file (expand-file-name "bible-page.html"
-;; 					   (temporary-file-directory))))
-
-;;     ;; Check if the file already exists in /tmp
-;;     (if (file-exists-p output-file)
-;; 	;; If it exists, play it directly
-;; 	(progn
-;;           (message "Playing %s %s..." book input)
-;;           (if (fboundp 'emms-play-file)
-;;               (emms-play-file output-file)
-;;             (start-process "mplayer" nil "mplayer" output-file)))
-
-;;       ;; If it doesn't exist, download it
-;;       ;; Download the page first
-;;       (call-process-shell-command
-;;        (format "curl -s '%s' > %s" audio-link temp-html-file))
-
-;;       ;; Use grep to extract the MP3 URL
-;;       (let* ((mp3-url (string-trim (shell-command-to-string
-;;                                     (format "grep -o 'https://stream.biblegateway.com/bibles/[^\"]*\\.mp3' %s | head -1"
-;;                                             temp-html-file)))))
-
-;; 	;; Remove the temporary HTML file
-;; 	(when (file-exists-p temp-html-file)
-;;           (delete-file temp-html-file))
-
-;; 	(if (string-match "^https://" mp3-url)
-;;             (progn
-;;               ;; Download the MP3 file using url-copy-file with status messages suppressed
-;;               (let ((url-show-status nil))
-;; 		(url-copy-file mp3-url output-file t))
-
-;;               ;; Play the file if it exists
-;;               (if (file-exists-p output-file)
-;;                   (progn
-;;                     (message "Playing %s %s..." book input)
-;;                     (if (fboundp 'emms-play-file)
-;; 			(progn
-;; 			  (require 'emms)
-;; 			  ;; (let ((emms-player-list-1 emms-player-list))
-;;                           (emms-play-file output-file))
-;;                       (start-process "mplayer" nil "mplayer" output-file)))
-;; 		(progn
-;; 		  (message "Failed to download audio for %s %s" book input))
-;; 		(message "No audio found for %s %s" book input))))))))
 
 (provide 'bible-gateway)
 ;;; bible-gateway.el ends here
